package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouProjectStrategy;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouProjectCreateCreativeService;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouProjectStrategyService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Api(tags = "快手-自动投放")
@RestController
@RequestMapping("/ai/projectCreate")
public class AiKuaishouProjectCreateCreativeController {
    @Autowired
    private IAiKuaishouProjectCreateCreativeService projectCreateCreativeService;
    @Autowired
    private IKuaishouProjectStrategyService kuaishouProjectStrategyService;
    @Autowired
    private IUserAllocationService userAllocationService;


    static ExecutorService executorService = Executors.newFixedThreadPool(5);

    /**
     * 项目自动创建
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/projectAutomaticCreates")
    public Result<Object> projectAutomaticCreates(String id, Integer operationType) {
        try {
            if (Check.isNull(id) || Check.isNull(operationType)) {
                throw new Exception("参数为空");
            }
            KuaishouProjectStrategy strategy = kuaishouProjectStrategyService.getById(id);
            if (Check.isNull(strategy)) {
                throw new Exception("根据id获取详细信息为空");
            }
            Long projectId = strategy.getProjectId();
            List<Long> list = userAllocationService.queryAutomaticAccounts(projectId);
            if (Check.isNull(list)) {
                return Result.ok("项目：" + projectId + "中未配置自动投放账户！");
            }
            list.forEach(accountId -> executorService.submit(() -> {
                Boolean unitOverrun = projectCreateCreativeService.getUnitOverrun(projectId.toString().concat(accountId.toString()));
                if (!unitOverrun) {
                    strategy.setAccountId(accountId);
                    // operationType 1检查反馈，2创建时信息
                    projectCreateCreativeService.projectAutomaticCreates(strategy, operationType);
                }
            }));
            return Result.ok("异步创建中...");
        } catch (Exception e) {
            log.error("项目自动创建异常", e.fillInStackTrace());
            return Result.error(e.getMessage());
        }
    }
}
